<?php

    class Create_Event_Chain_Action extends Reactor_Action
    {
        protected $tpl = 'create_event_chain.php';
        protected $name = 'Create a new event chain';
		
		public static $registration = array(
			ACTION_NAME => 'Create a new event chain',
			ACTION_CATEGORY => 'Nagios Reactor',
			ACTION_DESCRIPTION => 'Creates a new event chain as defined by a JSON file.',
			ACTION_AUTHOR => 'Jeremy Yunis, Nagios Enterprises, LLC',
			ACTION_VERSION => '1.0',
			ACTION_DATE => '02/18/2013', //Last version update
			ACTION_AUTHLEVEL => 0,  // 0 - everyone | 1 - Admin only
			ACTION_TYPE => 'core', //core | user. core modules can't be deleted
			ACTION_META => array(),	//Misc information	
		); 		
		
        protected $fields = array(
            'name' => array(
                'label' => 'name',
                'validate' => 'trim|required|min_length:10|max_length:255'
            ),
            'source' => array(
                'label' => 'source',
                'validate' => 'trim|required',
            ),
            'json' => array(
                'label' => 'chain json',
                'validate' => 'trim|xss_clean|prep_for_form|callback_check_json'
            ),
            'blocks' => array(
                'label' => 'global blocks',
                'validate' => 'xss_clean|callback_check_blocks'
            ),
            'context_var' => array(
                'label' => 'context variable',
                'validate' => 'trim|xss_clean|prep_for_form|alphanum'
            ),
        );

        public function get_label()
        {
            if($params = $this->get_params()) {
				$name=$params['name'];
				$source=$params['source'];
				$json=$params['json'];
				$blocks=$params['blocks'];
				
				$fstr="?";
				if($source=="json")
					$fstr="JSON";
				else
					$fstr="global blocks";
			
                return sprintf('Create a new event chain named \'%s\' from %s', $name,$fstr);
            }
            return parent::get_label();
        }
		
        public function check_blocks($blocks)
        {
            $source = $this->ci->input->post('source');
            $blocks = array_filter($blocks);
            if($source == 'global_blocks') {
                if(!count($blocks)) {
                    $this->ci->form_validation->set_message('check_blocks', _('Please choose at least one block for this chain.'));
                    return FALSE;
                }
                foreach($blocks as $block) {
                    if(!is_numeric($block)) {
                        $this->ci->form_validation->set_message('check_blocks', _('Please choose at least one block for this chain.'));
                        return FALSE;
                    }
                }
            }
            return TRUE;
        }

        public function check_json($str)
        {

            $source = $this->ci->input->post('source');
            if($source == 'json') {
                if(!strlen($str)) {
                    $this->ci->form_validation->set_message('check_json', _('This field is required.'));
                    return FALSE;
                }
            }
            return TRUE;
        }

        public function run($context)
        {
            $this->set_context($context);
            $params = $this->get_params();
            $name = sprintf("(%s) %s", _("auto"), $params['name']);

            if($params['source'] == 'json') {
                $this->log('info', 'Attempting to import JSON.');
                $json = $params['json'];
                if($new_chain = $this->ci->event_chain_model->create_from_json($json, $name=$name)) {
                    if(array_key_exists('context_var', $params)) {
                        $dest_context_var = $params['context_var'];
                        $context[$dest_context_var] = $new_chain->slug;
                    }
                    $this->set_result(0);
                    return $context;
                }
                $this->set_result(1);
            } elseif($params['source'] == 'global_blocks') {
                $this->log('info', 'Creating chain from global blocks.');
                $run = $this->get_run();
                $description = sprintf(_("Auto-created by event chain %s (run %s)"), $run->get_event_chain()->name,
                    $run->id);
                $chain_info = array(
                    'name' => $name,
                    'description' => $description,
                    'active' => TRUE
                );
                if($chain = $this->ci->event_chain_model->create($chain_info)) {
                    foreach($params['blocks'] as $tpl_id) {
                        $chain->add_node_from_tpl($tpl_id);
                    }
                    if(array_key_exists('context_var', $params)) {
                        $dest_context_var = $params['context_var'];
                        $context[$dest_context_var] = $chain->slug;
                    }

                    $this->set_result(0);
                    return $context;
                }
                $this->set_result(1);
            }
            return $context;
        }
    }
